package com.cmkg.cmkg.repository;

import com.cmkg.cmkg.node.disease;
import org.springframework.data.neo4j.annotation.Query;
import org.springframework.data.neo4j.repository.Neo4jRepository;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;


@Repository
public interface QuestionRespository extends Neo4jRepository<disease,Long>{

    /**
     * param疾病铭恒
     *
     * return 返回疾病的id
     */
    @Query("match(n:Disease) where n.name={name} return n.did")
    public Long getDiseaseId(@Param("name") String name);

    /**
     * param the name of medicine
     *
     * return the id of medicine
     */
    @Query("match(n:Yangsheng) where n.name={name} return n.yid")
    public Long getMedicineId(@Param("name") String name);

    /**
     * 对应问题模板 nm（病症） 症状
     *
     * param 疾病名称
     *
     * return 返回疾病的症状
     */
    @Query("match(n:Disease) where n.name={name} return n.syptom")
    public String getDiseaseSymptom(@Param("name") String name);

    /**
     * 对应问题模板 nm 治疗方法
     *
     * param nm 病症名称
     *
     * return 对应的药
     */
    @Query("match (n)-[r:my]-(b) where n.name={name} return b.name limit 10")
    public String[] getDiseaseToMedicine(@Param("name") String name);

    /**
     * 对应模板问题 nnt 用法
     *
     * param nnt 药品或养生方法名称
     *
     * return 药品用法
     */
    @Query("match(n:Yangsheng) where n.name={name} return n.operation")
    public String getMedicineOperation(@Param("name") String name);

    /**
     * 对应模板问题 nnt 作用
     *
     * param nnt 药品或养生方法名称
     *
     * return 药品作用
     */

    @Query("match(n:Yangsheng) where n.name={name} return n.symptom")
    public String getMedicineSyndromes(@Param("name") String name);

}
