package com.cmkg.cmkg.server.impl;

import com.cmkg.cmkg.Util.CodeMessage;
import com.cmkg.cmkg.process.ModelProcess;
import com.cmkg.cmkg.repository.QuestionRespository;
import com.cmkg.cmkg.server.QuestionService;
import com.cmkg.cmkg.server.ReturnD.funcDisea;
import com.cmkg.cmkg.server.ReturnData;
import com.hankcs.hanlp.dictionary.CustomDictionary;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;

import java.io.*;
import java.util.ArrayList;


@Service
@Primary
public class QuestionServiceImpl implements QuestionService {

    @Value("${rootDirPath}") //路径名需要更改
    private String rootDictPath;

    @Value("${HanLP.CustomDictionary.path.disease}")
    private String diseasePath;

    @Value("${HanLP.CustomDictionary.path.medicine}")
    private String medicinePath;

    @Autowired
    private QuestionRespository questionRepository;



    @Override
    public ReturnData answer(String question) throws Exception {

        ModelProcess queryProcess = new ModelProcess(rootDictPath);

        /**
         * 加载自定义的病症字典 == 设置词性 nm 0
         */

        loadDisease(diseasePath);

        /**
         * 加载自定义的药品字典 == 设置词性 nhm 0
         */
        loadMedicine(medicinePath);

        ArrayList<String> reStrings = queryProcess.analyQuery(question);
        for(String sss:reStrings) System.out.println(sss);
        int modelIndex = Integer.valueOf(reStrings.get(0));
        ReturnData answer = new ReturnData();
        String title = "";
        /**
         * 匹配问题模板
         */
        switch (modelIndex) {
            case 0:
                /**
                 * nm 治疗方法 == 病的治疗方法
                 */
                title = reStrings.get(1);
                System.out.println(title);
                String[] measure = questionRepository.getDiseaseToMedicine(title);

                if (measure.length==0) {
                    answer.setCode(CodeMessage.NoMessage);
                    answer.setMsg("暂时没有找到的消息");
                    answer.setData("");
                } else {
                    answer.setCode(CodeMessage.DiseaseToMedicine);
                    answer.setMsg(title);
                    answer.setData(measure);
                }
                break;
            case 1:
                /**
                 * nm 症状 == 病的症状
                 */
                title = reStrings.get(1);
                String symptom = questionRepository.getDiseaseSymptom(title);
                System.out.println(symptom);
                if (symptom != null) {
                    answer.setCode(CodeMessage.DiseaseToSymptom);
                    answer.setMsg(title);
                    answer.setData(symptom);
                } else {
                    answer.setCode(CodeMessage.NoMessage);
                    answer.setMsg("暂时没有找到的消息");
                    answer.setData("");
                }
                break;
            case 2:
                /**
                 * nhm 用法 == 药品用法
                 */
                title = reStrings.get(1);
                String use = questionRepository.getMedicineOperation(title);
                System.out.println(use);
                if (use != null) {
                    answer.setCode(CodeMessage.MedicineToOperation);
                    answer.setMsg(title);
                    answer.setData(use);
                } else {
                    answer.setCode(CodeMessage.NoMessage);
                    answer.setMsg("暂时没有找到的消息");
                    answer.setData("");
                }
                break;
            case 3:
                /**
                 * nhm 作用 == 药品作用
                 */
                title = reStrings.get(1);
                funcDisea funcDisea = new funcDisea();
                String function= questionRepository.getMedicineSyndromes(title);
                String[] disease = questionRepository.getDiseaseToMedicine(title);

                System.out.println(function);
                if (function != null) {
                    answer.setCode(CodeMessage.MedicineToSymptom);
                    answer.setMsg(title);
                    funcDisea.setDisease(disease);
                    funcDisea.setFunction(function);


                    answer.setData(funcDisea);
                } else {
                    answer.setCode(CodeMessage.NoMessage);
                    answer.setMsg("暂时没有找到的消息");
                    answer.setData("");
                }
                break;

        }

        System.out.println(answer);
        return answer;
    }

    @Override
    public void showDictPat() {
        System.out.println("HanLP分词字典及自定义问题模板根目录：" + rootDictPath);
        System.out.println("用户自定义扩展词库【病症名】：" + diseasePath);
    }

    /**
     * 加载自定义病症字典
     *
     * @param path
     */
    public void loadDisease(String path) {

        File file = new File(path);
        BufferedReader br = null;
        try {
            br = new BufferedReader(new FileReader(file));
            addCustomDictionary(br, 0);
        } catch (FileNotFoundException e1) {
            e1.printStackTrace();
        }

    }

    /**
     * 加载自定义药品类别字典
     *
     * @param path
     */
    public void loadMedicine(String path) {

        File file = new File(path);
        BufferedReader br = null;
        try {
            br = new BufferedReader(new FileReader(file));
            addCustomDictionary(br, 1);
        } catch (FileNotFoundException e1) {
            e1.printStackTrace();
        }
    }


    /**
     * 添加自定义分词及其词性，注意数字0表示频率，不能没有
     *
     * @param br
     * @param type
     */
    public void addCustomDictionary(BufferedReader br, int type) {

        String word;
        try {
            while ((word = br.readLine()) != null) {
                switch (type) {
                    /**
                     * 设置病症名词词性 == nm 0
                     */
                    case 0:
                        CustomDictionary.add(word, "nm 0");
                        break;
                    /**
                     * 设置药品名词词性 == nhm 0
                     */
                    case 1:
                        CustomDictionary.add(word, "nhm 0");
                        break;
                    default:
                        break;
                }
            }
            br.close();
        } catch (NumberFormatException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}